@@ -1,11 +1,11 @@
 gnm <- function(formula, eliminate = NULL, constrain = NULL, family = gaussian,
                 data = NULL, subset, weights, na.action,  method = "gnmFit",
-                offset, start = NULL, tolerance = 1e-4, iterStart = 2,
+                offset, start = NULL, tolerance = 1e-6, iterStart = 2,
                 iterMax = 500, trace = FALSE, verbose = TRUE, model = TRUE,
-                x = FALSE, termPredictors = FALSE, ...) {
-    
+                x = FALSE, termPredictors = FALSE, lsMethod = "qr", ...) {
+
     call <- match.call()
-    
+
     modelTerms <- gnmTerms(formula, substitute(eliminate), data)
     modelData <- match.call(expand.dots = FALSE)
     argPos <- match(c("data", "subset", "weights", "na.action", "offset"),
@@ -14,6 +14,11 @@ gnm <- function(formula, eliminate = NULL, constrain = NULL, family = gaussian,
                            formula = modelTerms,
                            as.list(modelData)[argPos],
                            drop.unused.levels = TRUE))
+    if (inherits(data, "table") && !is.empty.model(modelTerms)) {
+        xFactors <- modelData
+        xFactors$formula <- Freq ~ .
+        xFactors <- eval(xFactors, parent.frame())[, -1]
+    }
     modelData <- eval(modelData, parent.frame())
 
     if (!missing(eliminate)) {
@@ -21,9 +26,10 @@ gnm <- function(formula, eliminate = NULL, constrain = NULL, family = gaussian,
         if (!is.factor(Elim))
             stop("variables in 'eliminate' formula must be factors")
         nElim <- nlevels(Elim)
+        if (missing(lsMethod)) lsMethod <- "chol"
     }
     else nElim <- 0
-    
+
     if (method == "model.frame") {
         attr(modelData, "terms") <- attr(modelTerms, "terms")
         return(modelData)
@@ -34,7 +40,7 @@ gnm <- function(formula, eliminate = NULL, constrain = NULL, family = gaussian,
                 call. = FALSE)
         method <- "gnmFit"
     }
-    
+
     y <- model.response(modelData, "numeric")
     nObs <- NROW(y)
 
@@ -47,17 +53,15 @@ gnm <- function(formula, eliminate = NULL, constrain = NULL, family = gaussian,
     if (is.null(offset))
         offset <- rep.int(0, nObs)
 
-    if (!missing(family) && !exists(as.character(call$family)))
-        stop("family object \"", as.character(call$family), "\" not found")
-    if (is.character(family)) 
+    if (is.character(family))
         family <- get(family, mode = "function", envir = parent.frame())
-    if (is.function(family)) 
+    if (is.function(family))
         family <- family()
     if (is.null(family$family)) {
         print(family)
         stop("`family' not recognized")
     }
-    
+
     if (family$family == "binomial") {
         if (is.factor(y) && NCOL(y) == 1)
             y <- y != levels(y)[1]
@@ -163,13 +167,14 @@ gnm <- function(formula, eliminate = NULL, constrain = NULL, family = gaussian,
         }
         if (method == "model.matrix") return(X)
 
-        if (!is.numeric(tolerance) || tolerance <= 0) 
+        if (!is.numeric(tolerance) || tolerance <= 0)
             stop("value of 'tolerance' must be > 0")
-        if (!is.numeric(iterMax) || iterMax <= 0) 
+        if (!is.numeric(iterMax) || iterMax <= 0)
             stop("maximum number of iterations must be > 0")
-        
+
         if (onlyLin) {
             if (any(is.na(start))) start <- NULL
+            if (verbose) cat("Linear predictor - using glm.fit")
             fit <- glm.fit(X, y, family = family, weights = weights,
                            offset = offset, start = start,
                            control = glm.control(tolerance, iterMax, trace),
@@ -188,7 +193,8 @@ gnm <- function(formula, eliminate = NULL, constrain = NULL, family = gaussian,
         else
             fit <- gnmFit(modelTools, y, constrain, nElim, family, weights,
                           offset, nObs, start, tolerance, iterStart, iterMax,
-                          trace, verbose, x, termPredictors)
+                          trace, verbose, x, termPredictors,
+                          lsMethod = lsMethod)
     }
     if (is.null(fit)) {
         warning("Algorithm failed - no model could be estimated", call. = FALSE)
@@ -203,14 +209,13 @@ gnm <- function(formula, eliminate = NULL, constrain = NULL, family = gaussian,
 
     asY <- c("predictors", "fitted.values", "residuals", "prior.weights",
              "weights", "y", "offset")
-    if (inherits(data, "table") && !is.empty.model(modelTerms)) {
-        fit[asY] <- lapply(fit[asY], replace,
-                           list = as.numeric(names(fit$y)), x = data)
+    if (exists("xFactors", inherits = FALSE)) {
+        fit[asY] <- lapply(fit[asY], tapply, xFactors, sum)
         if (!is.null(fit$na.action)) fit$na.action <- NULL
     }
     else
         fit[asY] <- lapply(fit[asY], structure, names = names(y))
-    
+
     if (model) {
         attr(modelData, "terms") <- attr(modelTerms, "terms")
         fit$model <- modelData
